/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.model;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 * UnMarshaller to unMarshall iso 11783-10 messages
 *
 * @author Merijn Zengers & Gerben Feenstra
 */
@NbBundle.Messages({
    "isobus.error.isobus_unmarshaller.exception=The isobus file {0} could not be read!",
    "isobus.error.isobus_unmarshaller.timelog_not_found=File {0} cannot be found!",
    "isobus.error.isobus_unmarshaller.xml_cannot_be_null=the XMLFile provided cannot be null"
})
public class IsobusMarshaller {

    private static final AppLogger APP_LOGGER = AppLogFactory.getLogger(IsobusMarshaller.class);
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model");
    private final String PACKAGE_NAME = "nl.cloudfarming.client.isobus.model";
    private static final String DEFAULT_TASKDATA_FILENAME = "TASKDATA";
    private static final String DEFAULT_TASKDATA_FILENAME_EXT = "XML";

    /**
     * Marshalls the provided IsobusTaskData to the supplied FileObject
     *
     * @param targetFolder the fileObject to write to
     * @param isobusTaskData the IsobusTaskData to marshall
     */
    public void marshall(FileObject targetFolder, IsobusTaskData isobusTaskData) {
        JAXBContext jc;

        try {
            jc = JAXBContext.newInstance(PACKAGE_NAME);
            Marshaller marshaller = jc.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            // remove old file first if it exists:
            FileObject targetFile = targetFolder.getFileObject(DEFAULT_TASKDATA_FILENAME, DEFAULT_TASKDATA_FILENAME_EXT);
            if (targetFile != null) {
                targetFile.delete();
            }
            targetFile = targetFolder.createData(DEFAULT_TASKDATA_FILENAME, DEFAULT_TASKDATA_FILENAME_EXT);
            try (OutputStream outputStream = targetFile.getOutputStream()) {
                marshaller.marshal(isobusTaskData, outputStream);
            }
        } catch (IOException | JAXBException ex) {
            APP_LOGGER.error("Failed to marshall Isobus taskdata to xml");
            LOGGER.log(Level.SEVERE, "Failed to marshall Isobus taskdata to xml {0}", ex.getMessage());
        }
    }

    /**
     * UnMarshal File containing XML to an IsobusTaskData object
     *
     * @param xmlFile The File object which contains the XML
     * @return An IsobusTaskData object
     */
    public IsobusTaskData unMarshall(FileObject xmlFile) throws IllegalArgumentException {

        // XML should not be null
        if (xmlFile == null) {
            throw new IllegalArgumentException(NbBundle.getMessage(IsobusMarshaller.class, "isobus.error.isobus_unmarshaller.xml_cannot_be_null"));
        }

        // Unmarshall XML
        IsobusTaskData isobusTaskData = null;
        try {
            isobusTaskData = unMarshallIsobusTaskData(xmlFile.getInputStream());
        } catch (FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return isobusTaskData;
    }

    /**
     * Unmarshall the specified InputStream to an IsobusTaskData object. Do not
     * use, PROTECTED for testing only!
     *
     * @param taskdata Taskdata as inputstream
     * @return IsobusTaskData Unmarshalled isobus taskdata object
     */
    protected IsobusTaskData unMarshallIsobusTaskData(InputStream taskdata) {
        JAXBContext jc;

        try {
            jc = JAXBContext.newInstance(PACKAGE_NAME);
            Unmarshaller unmarshaller = jc.createUnmarshaller();
            IsobusTaskData isobusTaskData = (IsobusTaskData) unmarshaller.unmarshal(taskdata);
            return isobusTaskData;
        } catch (JAXBException ex) {
            APP_LOGGER.error("Failed to convert xml", ex);
        }
        return null;
    }
}
